//*******************************************************
//
//	Z80EM1
//
//	z80emuの I/F
//
//	2014.9.23	KAWAKAMI Yukio
//*******************************************************

#include	"hw_config.h"
#include	"main.h"
#include	"com.h"
#include	"timer.h"
#include	"ssprintf.h"

#include	"call_z80.h"

uint8_t	Rom[2048];
uint8_t Ram[1024];


//*******************************************************
//	M1 byte読み込み
//*******************************************************
uint8_t z80_fetch_byte(uint16_t adrs){

	if (adrs & 0x8000){
		return Ram[adrs & (sizeof(Ram)-1)];
	} else {
		return Rom[adrs & (sizeof(Rom)-1)];
	}
}


//*******************************************************
//	M1 word読み込み
//*******************************************************
uint16_t z80_fetch_word(uint16_t adrs){
	uint16_t	data;

	if (adrs & 0x8000){
		adrs &= (sizeof(Ram)-1);
		data = Ram[adrs++];
		data |= ((uint16_t)Ram[adrs]) << 8;
	} else {
		adrs &= (sizeof(Rom)-1);
		data = Rom[adrs++];
		data |= ((uint16_t)Rom[adrs]) << 8;
	}

	return data;
}


//*******************************************************
//	byte読み込み
//*******************************************************
uint8_t z80_read_byte(uint16_t adrs){

	if (adrs & 0x8000){
		return Ram[adrs & (sizeof(Ram)-1)];
	} else {
		return Rom[adrs & (sizeof(Rom)-1)];
	}
}


//*******************************************************
//	word読み込み
//*******************************************************
uint16_t z80_read_word(uint16_t adrs){

	uint16_t	data;

	if (adrs & 0x8000){
		adrs &= (sizeof(Ram)-1);
		data = Ram[adrs++];
		data |= ((uint16_t)Ram[adrs]) << 8;
	} else {
		adrs &= (sizeof(Rom)-1);
		data = Rom[adrs++];
		data |= ((uint16_t)Rom[adrs]) << 8;
	}

	return data;
}


//*******************************************************
//	byte書き込み
//*******************************************************
void z80_write_byte(uint16_t adrs, uint8_t data){

	if (adrs & 0x8000){
		Ram[adrs & (sizeof(Ram)-1)] = data;
	}
}


//*******************************************************
//	word書き込み
//*******************************************************
void z80_write_word(uint16_t adrs, uint16_t data){

	if (adrs & 0x8000){
		adrs &= (sizeof(Ram)-1);
		Ram[adrs++] = (uint8_t)data;
		Ram[adrs] = (uint8_t)(data >> 8);
	}
}


//*******************************************************
//	IO 入力
//*******************************************************
uint8_t z80_input_byte(uint16_t port){
	uint8_t	data;

	DATA_DIR_IN;
	ADRS_OUT(port);
	IORQ_ON;
	RD_ON;

	NOP;
	NOP;
	NOP;
	NOP;	// 4つで 240ns @48MHz
	NOP;
	NOP;
	NOP;
	NOP;	// 8つで 370ns @48MHz
	NOP;
	NOP;
	NOP;
	NOP;	// 12で 460ns @48MHz

	data = DATA_IN;
	RD_OFF;
	IORQ_OFF;
	return data;
}


//*******************************************************
//	IO 出力
//*******************************************************
void z80_output_byte(uint16_t port, uint8_t data){

	ADRS_OUT(port);
	DATA_DIR_OUT;
	DATA_OUT(data);
	IORQ_ON;
	WR_ON;

	NOP;
	NOP;
	NOP;
	NOP;	// 4つで 240ns @48MHz
	NOP;
	NOP;
	NOP;
	NOP;	// 8つで 370ns @48MHz
	NOP;
	NOP;
	NOP;
	NOP;	// 12で 460ns @48MHz
	
	WR_OFF;
	IORQ_OFF;
	NOP;
	NOP;
	DATA_DIR_IN;
}


//*******************************************************
//	実ROM内容を仮想ROMエリアにコピー
//*******************************************************
void copy_ROM(void){
	uint16_t	adrs;
	int			i;

	DATA_DIR_IN;
	ADRS_DIR_OUT;
	for (adrs = 0; adrs < sizeof(Rom); adrs++){
		ADRS_OUT(adrs);
		MREQ_ON;
		RD_ON;

		NOP;
		NOP;
		NOP;
		NOP;	// 4つで 240ns @48MHz
		NOP;
		NOP;
		NOP;
		NOP;	// 8つで 370ns @48MHz
#if 0
		NOP;
		NOP;
		NOP;
		NOP;	// 12で 460ns @48MHz
		NOP;
		NOP;
		NOP;
		NOP;	// 16で 580ns @48MHz
#endif

		Rom[adrs] = DATA_IN;
		RD_OFF;
		MREQ_OFF;
	}
}
