//*******************************************************
//
//	Z80EM1
//
//	2014.9.24	KAWAKAMI Yukio
//*******************************************************

#include	"hw_config.h"
#include	"main.h"
#include	"com.h"
#include	"timer.h"
#include	"term.h"
#include	"ssprintf.h"
#include	"flash.h"

#include	"call_z80.h"

Z80_STATE Z80st;

CONFIG	Config;
extern COM_WORK	Com0;

extern __IO uint16_t Timer;

__IO int		IntCount;
__IO uint8_t	NMIflag;

const char *BootMsg = "\n" TARGET_NAME " Version " VERSION "\n" \
	"Build Date : " __DATE__ " " __TIME__ "\n";

extern uint8_t Rom[2048];

int	InstCount;

//********************************************************
//	ND80Z NMI(GPIO0_6)割り込み
//	約640Hz
//********************************************************
void PIOINT0_IRQHandler(void){

//	++IntCount;

	NMIflag = 1;

	LPC_GPIO0->IC |= (1<<6);
}


//********************************************************
//	BOD 割り込み
//********************************************************
void BOD_IRQHandler(void){

	// 無限ループ
	for (;;){
		WDR;
		SLEEP;
	}
}


//********************************************************
//	WDT初期化
//********************************************************
static void wdt_init(int n){

	LPC_SYSCON->SYSAHBCLKCTRL |= (1 << 15);	// WDT clock
	LPC_SYSCON->WDTOSCCTRL = (1 << 5)|0;	// 0.6MHz / 2 = 300kHz
	LPC_SYSCON->PDRUNCFG &= ~(1 << 6);		// WDTOSC_PDクリア

	LPC_SYSCON->WDTCLKDIV = 300 / 4;	// TCが約1ms単位になるよう設定
	LPC_SYSCON->WDTCLKSEL = 0x02;		// Select watchdog osc
	LPC_SYSCON->WDTCLKUEN = 0x01;		// Update clock
	LPC_SYSCON->WDTCLKUEN = 0x00;		// Toggle update register once
	LPC_SYSCON->WDTCLKUEN = 0x01;
	while (!(LPC_SYSCON->WDTCLKUEN & 0x01));	// Wait until updated

	LPC_WDT->TC = n;	// 約1ms単位で指定
	LPC_WDT->MOD = WDEN | WDRESET;	// チップリセット
	// 以降、変更不可
	WDR;
}


//********************************************************
//	main
//********************************************************
void main(void){
	int	i;

	/* SystemInit was already executed in asm startup */

	__disable_irq();

	//LPC_SYSCON->SYSMEMREMAP = 2;	// User Flash Mode
	{	// RAMへベクタコピー
		uint32_t	*src, *dst;
		src = (uint32_t *)APPLICATION_START_ADDRESS;
		dst = (uint32_t *)RAM_START_ADDRESS;
		for (i = 0; i < 48; i++){
			*dst++ = *src++;
		}
	}
	LPC_SYSCON->SYSMEMREMAP = 1;	// User RAM Mode

	LPC_SYSCON->BODCTRL = (1 << 4)|(0 << 2)|(3 << 0);	// 2.6V RESET

	//LPC_SYSCON->BODCTRL = (3 << 2)|(3 << 0);	// 2.8V BOD interrupt
	//NVIC_EnableIRQ(BOD_IRQn);

	port_init();
	SysTickInit(1000);	// 1ms
	wdt_init(3000);		// 約3秒

	__enable_irq();

	wait_ms(100);

	load_conf(&Config);
	if (Config.mark != CONFIG_MARK){
		init_conf(&Config);
		save_conf(&Config);
	}

	init_com(&Com0, 115200);
	cputs(BootMsg);

	// 実ROMの内容を仮想ROMへコピー
	copy_ROM();
#if 0
	{
		char	msg[32];
		int		adrs;
		for (adrs = 0; adrs < 128; adrs++){
			if ((adrs & 15) == 0){
				ssprintf(msg, "%04X:", adrs);
				cputs(msg);
			}
			ssprintf(msg, "%02X ", Rom[adrs]);
			cputs(msg);
			if ((adrs & 15) == 15){
				cputs("\n");
			}
		}
	}
#endif

	Z80Reset(&Z80st);
	
	// メインループ
	for (;;){
		WDR;

		Timer = 20;
		InstCount = 0;
		LED_ON;
		while(Timer){
			if (NMIflag){
				NMIflag = 0;
				Z80NonMaskableInterrupt(&Z80st);
			}
			Z80Emulate(&Z80st, Config.exec_inst);
			InstCount += Config.exec_inst;
		}
		LED_OFF;

		job_callback();
	}
}


//********************************************************
//	待ち時間に実行するコールバック
//********************************************************
void job_callback(void){

	job_term();

	if (IS_BOOT){
		boot(BOOT_MARK);
	}
}


//********************************************************
//	ブートローダ起動
//********************************************************
void boot(uint32_t mark){
	typedef void (*func)(void);
	func		jump_to_bootloader;
	uint32_t	*jump_adrs;
	uint32_t	*p;
	uint32_t	sp;

	wait_send_com(&Com0);
	
	__disable_irq();	// 割り込み禁止

	// WWDT停止
	LPC_WDT->MOD = 0;

	// 全割り込み無効
	NVIC->ICER[0] = 0xFFFFFFFF;
	NVIC->ICPR[0] = 0xFFFFFFFF;

	LPC_SYSCON->SYSMEMREMAP = 2;	// User Flash Mode

	// ブートローダ起動マーク
	p = (uint32_t *)RAM_START_ADDRESS;
	*p = mark;
	
	// boot_jump
	p = (uint32_t *)BOOTLOADER_ADDRESS;
	sp = *p++;
	jump_adrs = (u32 *)(*p);
	jump_to_bootloader = (func)jump_adrs;
	__set_MSP(sp);
	jump_to_bootloader();

	//asm volatile ("mov r0, %0"::"r"(adrs));
	//asm volatile ("ldr sp, [r0]"::);
	//asm volatile ("ldr pc, [r0, #4]"::);
}


//********************************************************
//	CONFIG読み込み
//********************************************************
void load_conf(CONFIG *config){

	memcpy((void *)config, (void *)CONFIG_FLASH_ADDRESS, sizeof(CONFIG));
}


//********************************************************
//	CONFIG保存
//********************************************************
void save_conf(CONFIG *config){
	uint8_t	buff[FLASH_PAGE_SIZE];

	memset(buff, 0xFF, FLASH_PAGE_SIZE);
	memcpy(buff, (void *)config, sizeof(CONFIG));
	write_flash(buff, CONFIG_FLASH_ADDRESS);
}


//********************************************************
//	CONFIG初期化
//********************************************************
void init_conf(CONFIG *config){

	memset((void *)config, 0, sizeof(CONFIG));
	config->prog_mark = 0xFF;
	config->exec_inst = 24;
	config->mark = CONFIG_MARK;
}


//********************************************************
//	GPIO等初期化
//********************************************************
void port_init(void){

	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<6);

	LPC_IOCON->R_PIO1_1  &= ~0x07;	// JTAG Disable
	LPC_IOCON->R_PIO1_1  |= 0x01;

	CLK_INIT;
	NMI_INIT;
	LED_INIT;
	INT_INIT;
	WAIT_INIT;
	RFSH_INIT;
	M1_INIT;
	MREQ_INIT;
	IORQ_INIT;
	RD_INIT;
	WR_INIT;
	DATA_INIT;
	ADRS_INIT;

	// ND80Z NMI(GPIO0_6)割り込み
	LPC_GPIO0->IS &= ~(1<<6);
	LPC_GPIO0->IBE &= ~(1<<6);
	NVIC_EnableIRQ(EINT0_IRQn);
	LPC_GPIO0->IE = (1<<6);
}



