//*******************************************************
//
//	Z80EM1
//
//	シリアル端末操作
//
//	2014.9.24	KAWAKAMI Yukio
//*******************************************************

#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"hw_config.h"
#include	"main.h"
#include	"com.h"
#include	"term.h"
#include	"ssprintf.h"
#include	"flash.h"
#include	"timer.h"

#include	"call_z80.h"

extern CONFIG	Config;
extern COM_WORK	Com0;
extern const char *BootMsg;

extern __IO int	IntCount;

extern Z80_STATE Z80st;
extern int InstCount;

//*******************************************************
//	クロック情報表示
//*******************************************************
void print_clockinfo(void){
	char	msg[48];

	ssprintf(msg, "Clock : %dMHz\n", SystemCoreClock / 1000000);
	cputs(msg);
	cputs("Clock Source for main clock : ");
	switch (LPC_SYSCON->MAINCLKSEL & 0x03){
	  case 0:	cputs("IRC oscillator\n");	break;
	  case 1:	cputs("Input clock to system PLL\n");	break;
	  case 2:	cputs("WDT oscillator\n");	break;
	  case 3:	cputs("System PLL clock out\n");	break;
	}
	if (LPC_SYSCON->MAINCLKSEL & 0x01){
		cputs("PLL Source : ");
		switch (LPC_SYSCON->SYSPLLCLKSEL & 0x03){
		  case 0:	cputs("IRC oscillator\n");	break;
		  case 1:	cputs("System oscillator\n");	break;
		}
	}
}


//*******************************************************
//	デバッグ端末操作
//*******************************************************
void job_term(void){
	static uint8_t	com_buff[80];
	static int		com_buff_p = 0;

	while(check_com(&Com0)){
		u8		c;
		COM_ARG	arg;
		char	msg[32];

		c = read_com(&Com0);

		if (c == '\r'){	// CR

			split_arg(com_buff, &arg);	// 引数分割
			if (strcmp(arg.arg0, "VER") == 0){
				cputs(BootMsg);
			} else if ((strcmp(arg.arg0, "BOOTLOADER") == 0)||
					   (strcmp(arg.arg0, "BOOT") == 0)){
				wait_send_com(&Com0);
				boot(BOOT_MARK);
			} else if ((strcmp(arg.arg0, "SPM_PAGESIZE") == 0)||
					   (strcmp(arg.arg0, "FLASH_PAGESIZE") == 0)){
				ssprintf(msg, "%d\n", FLASH_PAGE_SIZE);
				cputs(msg);
			} else if (strcmp(arg.arg0, "UPLOAD") == 0){
				write_com(&Com0, '*');
			} else if (strcmp(arg.arg0, "CLOCK") == 0){
				print_clockinfo();
			} else if (strcmp(arg.arg0, "CONFIG") == 0){
				strupr(arg.arg1);
				if (!arg.arg1[0]){
					ssprintf(msg, "exec_inst=%d\n", Config.exec_inst);
					cputs(msg);
				} else if (strcmp(arg.arg1, "SAVE") == 0){
					save_conf(&Config);
					cputs("OK\n");
				} else if (strcmp(arg.arg1, "LOAD") == 0){
					load_conf(&Config);
					cputs("OK\n");
				} else if (strcmp(arg.arg1, "INIT") == 0){
					init_conf(&Config);
					save_conf(&Config);
					cputs("OK\n");
				}
			} else if (strcmp(arg.arg0, "EXEC") == 0){
				strupr(arg.arg1);
				if (strcmp(arg.arg1, "INST") == 0){
					int	n;
					n = get_value(arg.arg2);
					if (n > 0){
						Config.exec_inst = n;
						cputs("OK\n");
					}
				}
			} else if (strcmp(arg.arg0, "DEBUG") == 0){
				int	n;
				ssprintf(msg, "status=%d\n", Z80st.status);
				cputs(msg);
				ssprintf(msg, "PC=%04Xh\n", Z80st.pc);
				cputs(msg);
				n = 20000 * 10 + InstCount / 2;
				n /= InstCount;
				ssprintf(msg, "CycleTime=%d.%duS\n", n/10, n%10);
				cputs(msg);
			} else if ((strcmp(arg.arg0, "DUMP") == 0)||
					   (strcmp(arg.arg0, "D") == 0)){	// メモリダンプ
				static uint32_t	adrs = 0;
				uint8_t	*p, c;
				int	i;
				if (arg.arg1[0] != 0){
					adrs = get_value(arg.arg1);
				}
				p = (uint8_t *)adrs;
				if (adrs < 0x4000){
					for (i = 0; i < 128; i++){
						if (i == 0){
							uint32_t	a;
							a = adrs & ~15;
							ssprintf(msg, "%08lX:", a);
							cputs(msg);
							while(a < adrs){
								cputs("   ");
								a++;
							}
						} else if((adrs & 15) == 0){
							ssprintf(msg, "%08lX:", adrs);
							cputs(msg);
						}
						c = *p++;
						ssprintf(msg, "%02X ", c);
						cputs(msg);
						if ((++adrs & 15) == 0){
							cputs("\n");
							wait_send_com(&Com0);
						}
					}
				} else {
					uint32_t	data;
					adrs &= ~3;
					data = *((uint32_t *)adrs);
					ssprintf(msg, "%08lX:%lX", adrs, data);
					cputs(msg);
				}
				cputs("\n");
			}
			com_buff[0] = 0;
			com_buff_p = 0;
		} else if (c == 8){		// BS
			if (com_buff_p > 0){
				--com_buff_p;
				com_buff[com_buff_p] = 0;
				cputs("\b \b");
			}
		} else if ((c < ' ')||(c >= 0x7F)){
			;	// 無視
		} else if (com_buff_p < (sizeof(com_buff) - 1)){
			// CR以外の文字入力
			com_buff[com_buff_p++] = c;
			com_buff[com_buff_p] = 0;
		}
	}
	
}


//*******************************************************
//	コマンド文字列を引数に分解
//*******************************************************
void split_arg(u8 *buff, COM_ARG *arg){
	char	*p, c;
	int		n;

	memset((void *)arg, 0, sizeof(COM_ARG));
	arg->buff = buff;

	p = (char *)buff;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		if ((c >= 'a')&&(c <= 'z'))		c &= 0xDF;	// 大文字化
		arg->arg0[n++] = c;
		arg->arg0[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg0)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	if (*p == 0)	return;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		arg->arg1[n++] = c;
		arg->arg1[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg1)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	if (*p == 0)	return;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		arg->arg2[n++] = c;
		arg->arg2[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg2)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
}


//*******************************************************
//	コマンド文字列の数値を値に変換
//*******************************************************
int get_value(char *str){
	int	n = 0;

	if ((strcmp(str, "ON") == 0)||
		(strcmp(str, "on") == 0)){
		n = 1;
	} else if ((strcmp(str, "OFF") == 0)||
			   (strcmp(str, "off") == 0)){
		;
	} else if ((*str == '$')||
			   ((*str >= 'A')&&(*str <= 'F'))||
			   ((*str >= 'a')&&(*str <= 'f'))||
			   (str[strlen(str)-1] == 'h')){
		if (*str == '$')	str++;
		while(is_hex((u8)*str)){
			n <<= 4;
			n += hex_val((u8)*str++);
		}
	} else if ((*str == '-')||
			   (isdigit((int)*str))){
		if (*str == '-'){
			n = -1;
			str++;
		} else {
			n = 1;
		}
		n *= atoi(str);
	}
	return n;
}


//*******************************************************
//	0-9 A-F a-f 判別
//*******************************************************
char is_hex(u8 c){

	if (c < '0')	return 0;
	if (c <= '9')	return 1;
	if (c < 'A')	return 0;
	if (c <= 'F')	return 1;
	if (c < 'a')	return 0;
	if (c <= 'f')	return 1;
	return 0;
}


//*******************************************************
//	16進数文字の数値変換
//*******************************************************
u8 hex_val(u8 c){

	if (c <= '9')	return c - '0';
	if (c <= 'F')	return c - 'A' + 10;
	if (c <= 'f')	return c - 'a' + 10;
	return 0;
}


//*******************************************************
//	空白文字スキップ
//*******************************************************
char *skip_space(char *p){
	char	c;

	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	return p;
}


//*******************************************************
//	次の文字列先頭までスキップ
//*******************************************************
char *skip_next_word(char *p){
	char	c;

	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		p++;
	}
	return skip_space(p);
}


