//*******************************************************
//
//	Z80EM1	Bootloader
//
//	2014.9.19	KAWAKAMI Yukio
//*******************************************************

#include	"hw_config.h"
#include	"main.h"
#include	"com.h"
#include	"timer.h"
#include	"term.h"
#include	"ssprintf.h"
#include	"flash.h"
#include	"program.h"

//#define	FORCE_BOOT		// ブートローダ強制起動

CONFIG Config;
extern COM_WORK	Com0;

int StartProgram = 0;

const char *BootMsg = "\n" TARGET_NAME " BOOTLOADER Version " VERSION "\n" \
	"Build Date : " __DATE__ " " __TIME__ "\n";


//********************************************************
//	アプリケーションブート
//********************************************************
void boot(uint32_t mark){
	typedef void (*func)(void);
	func		jump_to_app;
	uint32_t	jump_adrs;
	uint32_t	*p = (uint32_t *)APPLICATION_START_ADDRESS;
	uint32_t	sp;

	sp = *p++;
	jump_adrs = *p;
	jump_to_app = (func)jump_adrs;

	if ((jump_adrs > APPLICATION_START_ADDRESS)&&
		(jump_adrs < (APPLICATION_START_ADDRESS + FLASH_SIZE))){
		int	i;

		__disable_irq();	// 割り込み禁止

		// WDT停止
		LPC_WDT->MOD = 0;

		// 全割り込み無効
		for (i = 0; i < 8; i++){
			NVIC->ICER[i] = 0xFFFFFFFF;
			NVIC->ICPR[i] = 0xFFFFFFFF;
		}

		// boot_jump
		__set_MSP(sp);
		jump_to_app();

		//asm volatile ("mov r0, %0"::"r"(adrs));
		//asm volatile ("ldr sp, [r0]"::);
		//asm volatile ("ldr pc, [r0, #4]"::);
	}
}


//********************************************************
//	main
//********************************************************
void main(void){
	int	i;

	/* SystemInit was already executed in asm startup */

	__disable_irq();

	LPC_SYSCON->SYSMEMREMAP = 2;	// User Flash Mode

	LPC_SYSCON->BODCTRL = (1 << 4)|(0 << 2)|(3 << 0);	// 2.6V RESET
	port_init();
	SysTickInit(1000);	// 1ms

	load_conf(&Config);

#ifndef	FORCE_BOOT
	{	// アプリケーションとブートローダのどちらを起動するか判断
		uint32_t *p = (uint32_t *)RAM_START_ADDRESS;
		if ((Config.prog_mark == 0xFF)&&	// 書き込み途中でない
			((*p != BOOT_MARK)&&(*p != BOOT_MARK_PROGRAM))){ // BOOT命令でない
			boot(0);	// アプリケーションブート
		}
		if (*p == BOOT_MARK_PROGRAM)	StartProgram = 1;
		*p = 0;	// ブートマーククリア	リンカスクリプトで予約済みのこと
	}
#endif

	__enable_irq();

	wait_ms(100);

	init_com(&Com0, 115200);
	cputs(BootMsg);

	// メインループ
	for (;;){
		WDR;
		SLEEP;
		job_callback();

		if (StartProgram){
			StartProgram = 0;
			update_firm_com();
		}
	}
}


//********************************************************
//	待ち時間に実行するコールバック
//********************************************************
void job_callback(void){

	job_term();
}


//********************************************************
//	CONFIG読み込み
//********************************************************
void load_conf(CONFIG *config){

	memcpy((void *)config, (void *)CONFIG_FLASH_ADDRESS, sizeof(CONFIG));
}



//********************************************************
//	GPIO等初期化
//********************************************************
void port_init(void){

	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<6);

	LPC_IOCON->R_PIO1_1  &= ~0x07;	// JTAG Disable
	LPC_IOCON->R_PIO1_1  |= 0x01;

	CLK_INIT;
	NMI_INIT;
	LED_INIT;
	INT_INIT;
	WAIT_INIT;
	RFSH_INIT;
	M1_INIT;
	MREQ_INIT;
	IORQ_INIT;
	RD_INIT;
	WR_INIT;
	DATA_INIT;
	ADRS_INIT;
}


