//*******************************************************
//
//	Z80EM1	Bootloader
//
//	ファームウェアプログラム
//
//	2014.9.19	KAWAKAMI Yukio
//*******************************************************

#include	"hw_config.h"
#include	"main.h"
#include	"com.h"
#include	"timer.h"
#include	"ssprintf.h"
#include	"flash.h"
#include	"program.h"

extern COM_WORK	Com0;

extern __IO uint16_t WaitTimer;
extern __IO uint16_t AckTimer;


#define print(x)		// LCD表示なので無視

//*******************************************************
//	ブロック受信
//*******************************************************
int term_block_read(uint8_t *buff, uint16_t len, uint16_t tm){

	AckTimer = tm;

	while(AckTimer){
		if (check_com(&Com0)){
			*buff++ = read_com(&Com0);
			if (--len == 0){
				return 0;
			}
		}
	}
	return 1;
}


//********************************************************
//	エラー表示
//********************************************************
static void print_msg(char *msg){
	cputs(msg);
	print(msg);
}


//********************************************************
//	シリアルポート読み込み・ファームウェアプログラム
//********************************************************
int update_firm_com(void){
	TRANS_FILE	file;
	uint32_t	size;
	uint32_t	end_adrs;
	uint32_t	adrs = 0;
	uint32_t	page;
	uint8_t		buff[FLASH_PAGE_SIZE];
	int			i;
	char		msg[32];

	write_com(&Com0, ACK);
	
	// ファイル情報読み込み
	if (term_block_read((uint8_t *)&file, sizeof(file), 100)){
		return 0;
	}
	size = file.fsize;	// サイズ
	if ((size < 1024)||(size > 48 * 1024)){
		write_com(&Com0, NACK);
		print_msg("File size Error\n");
		return 1;
	}
	if (strcmp(file.fname, FIRM_FNAME) != 0){
		write_com(&Com0, NACK);	// ファイル名が異なる
		print_msg("File name Error\n");
		return 1;
	}

	end_adrs = size + APPLICATION_START_ADDRESS;
	end_adrs += FLASH_PAGE_SIZE - 1;
	end_adrs &= ~(FLASH_PAGE_SIZE - 1);	// 終了アドレスを FLASH_PAGE_SIZEに揃える

	memcpy(buff, (void *)CONFIG_FLASH_ADDRESS, FLASH_PAGE_SIZE);
	buff[0] = 'W';	// 書き換え中
	write_flash(buff, CONFIG_FLASH_ADDRESS);

	write_com(&Com0, ACK);
	
	WDR;
	page = adrs = APPLICATION_START_ADDRESS;
	for (i = 0; adrs < end_adrs; adrs++){

		if (size > 0){
			// タイムアウトチェック
			WaitTimer = 1000;	// 1秒
			while(WaitTimer){
				WDR;
				if (check_com(&Com0))	break;
			}
			if (WaitTimer == 0){
				write_com(&Com0, NACK);
				ssprintf(msg, "\n%X:Timeout\n", adrs);
				print_msg(msg);
				return 1;
			}
			buff[i++] = read_com(&Com0);
			--size;

#ifdef PACKET_SIZE
			if (((i % PACKET_SIZE) == 0)&&(i < FLASH_PAGE_SIZE)){
				// XBee同期の為にパケットごとに ACKを返す
				write_com(&Com0, '.');	// ACK
			}
#endif
		} else {
			buff[i++] = 0xFF;
		}

		if (i >= FLASH_PAGE_SIZE){
			if (write_page(page, buff)){
				write_com(&Com0, NACK);
				return 1;
			}
			write_com(&Com0, ACK);
			page += FLASH_PAGE_SIZE;
			i = 0;
		}
	}

	memcpy(buff, (void *)CONFIG_FLASH_ADDRESS, FLASH_PAGE_SIZE);
	buff[0] = 0xFF;	// 書き換え完了
	write_flash(buff, CONFIG_FLASH_ADDRESS);
	print("\nOK");
	cputs("\n");

	return 0;
}


#if 0
//********************************************************
//	microSD読み込み・ファームウェアプログラム
//********************************************************
int update_firm_sd(void){
	FIL			fp;
	FRESULT		res;
	uint32_t	size;
	uint32_t	end_adrs;
	uint32_t	page;
	uint8_t		buff[FLASH_PAGE_SIZE];
	char		msg[32];


	res = f_open(&fp, FIRM_FNAME, FA_READ | FA_OPEN_EXISTING);
	if (res != FR_OK){
		ssprintf(msg, "Disk Err:%d", res);
		print(msg);
		return 1;
	}
	
	// ファイル情報読み込み
	size = fp.fsize;	// サイズ
	if ((size < 1024)||(size > 48 * 1024)){
		print("File size Error\n");
		return 1;
	}

	end_adrs = size + APPLICATION_START_ADDRESS;
	end_adrs += FLASH_PAGE_SIZE - 1;
	end_adrs &= ~(FLASH_PAGE_SIZE - 1);	// 終了アドレスを FLASH_PAGE_SIZEに揃える

	memcpy(buff, (void *)CONFIG_FLASH_ADDRESS, FLASH_PAGE_SIZE);
	buff[0] = 'W';	// 書き換え中
	write_flash(buff, CONFIG_FLASH_ADDRESS);

	WDR;
	page = APPLICATION_START_ADDRESS;
	while(page < end_adrs){
		UINT	n;

		memset(buff, 0xFF, FLASH_PAGE_SIZE);
		res = f_read(&fp, buff, FLASH_PAGE_SIZE, &n);
		if (res != FR_OK){
			f_close(&fp);
			ssprintf(msg, "Disk Err:%d", res);
			print(msg);
			return 1;
		} else if (n > 0){
			if (write_page(page, buff)){
				return 1;
			}
			page += FLASH_PAGE_SIZE;
		}
		if (f_eof(&fp))	break;
	}
	f_close(&fp);

	memcpy(buff, (void *)CONFIG_FLASH_ADDRESS, FLASH_PAGE_SIZE);
	buff[0] = 0xFF;	// 書き換え完了
	write_flash(buff, CONFIG_FLASH_ADDRESS);
	print("\nOK");

	return 0;
}
#endif


//********************************************************
//	フラッシュ１ページ書き込み
//********************************************************
int write_page(uint32_t	adrs, uint8_t *buff){
	char	msg[32];
	int		retry;

	ssprintf(msg, "\rwriting:%X ", adrs);
	print(msg);

	for (retry = 0; retry < 3; retry++){
		WDR;
		write_flash(buff, adrs);
		if (verify_flash(buff, adrs) == 0)	break;
	}
	if (retry >= 3){
		ssprintf(msg, "\n%X:Verify Error\n", adrs);
		print_msg(msg);
		return 1;
	}
	return 0;
}


