/******************************************************************************/
//
//	Z80EM1
//
/*!
	@file			hw_config.h
	@author         Nemui Trinomius (http://nemuisan.blog.bai.ne.jp)
    @version        2.00
    @date           2010.03.28
	@brief          For LPC1xxx LPCXPresso/MARY Compatible Circuit.

    @section HISTORY
		2010.07.21	V1.00	Starts here.
		2011.03.28  V2.00	Added MARY Board Support.

    @section LICENSE
		BSD License. See Copyright.txt
*/
/******************************************************************************/
#ifndef __HW_CONFIG_H
#define __HW_CONFIG_H 0x0200

#ifdef __cplusplus
 extern "C" {
#endif

/* General Inclusion */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include "LPC11xx.h"                        /* LPC11xx definitions */

#include "driver_config.h"
#include "wdt.h"
#include "gpio.h"

/* Function Inclusion */

/* Macros */
#define countof(a)   (sizeof(a) / sizeof(*(a)))

#define		NOP		asm volatile("nop\n"::)
//#define		WDR		IWDG_ReloadCounter()
#define		SLEEP	(LPC_PMU->PCON = 0x00, __WFI())
#define		WDR		(LPC_WDT->FEED = 0xAA, LPC_WDT->FEED = 0x55)

typedef	signed char		s8;
typedef signed short	s16;
typedef	signed long		s32;
typedef	unsigned char	u8;
typedef unsigned short	u16;
typedef	unsigned long	u32;

#define	GPIO_HIGH(p, b)		LPC_GPIO##p->MASKED_ACCESS[1<<b] = (1<<b)
#define	GPIO_LOW(p, b)		LPC_GPIO##p->MASKED_ACCESS[1<<b] = 0
#define GPIO_IS_HIGH(p, b)	(LPC_GPIO##p->MASKED_ACCESS[1<<b])
#define GPIO_IS_LOW(p, b)	(!(LPC_GPIO##p->MASKED_ACCESS[1<<b]))
	 
// GPIO
#define	IS_BOOT		GPIO_IS_LOW(0, 1)

#define	CLK_INIT	LPC_IOCON->PIO0_2 = 0x0F0;	\
	 LPC_GPIO0->DIR &= ~(1<<2)
#define	IS_CLK_HIGH	GPIO_IS_HIGH(0, 2)
#define	IS_CLK_LOW	GPIO_IS_LOW(0, 2)

#define	NMI_INIT	LPC_IOCON->PIO0_6 = 0x0F0;	\
	 LPC_GPIO0->DIR &= ~(1<<6)
#define	IS_NMI		GPIO_IS_LOW(0, 6)

#define	LED_ON		GPIO_HIGH(0, 7)
#define	LED_OFF		GPIO_LOW(0, 7)
#define	LED_INIT	LPC_IOCON->PIO0_7 = 0x0C0;	\
	 LPC_GPIO0->DIR |= (1<<7);	\
	 LED_OFF

#define	INT_INIT	LPC_IOCON->PIO0_8 = 0x0F0;	\
	 LPC_GPIO0->DIR &= ~(1<<8)
#define	IS_INT		GPIO_IS_LOW(0, 8)

#define	WAIT_INIT	LPC_IOCON->PIO0_9 = 0x0F0;	\
	 LPC_GPIO0->DIR &= ~(1<<9)
#define	IS_WAIT		GPIO_IS_LOW(0, 9)

#define	RFSH_ON		GPIO_LOW(0, 10)
#define	RFSH_OFF	GPIO_HIGH(0, 10)
#define	RFSH_INIT	LPC_IOCON->SWCLK_PIO0_10 = 0x0C1;	\
	 LPC_GPIO0->DIR |= (1<<10);	\
	 RFSH_OFF

#define	M1_ON		GPIO_LOW(0, 11)
#define	M1_OFF		GPIO_HIGH(0, 11)
#define	M1_INIT		LPC_IOCON->R_PIO0_11 = 0x0C1;	\
	 LPC_GPIO0->DIR |= (1<<11);	\
	 M1_OFF

#define	RXD_INIT	LPC_IOCON->PIO1_6 = 0x0F1
#define	TXD_INIT	LPC_IOCON->PIO1_7 = 0x0C1

#define	MREQ_ON		GPIO_LOW(1, 8)
#define	MREQ_OFF	GPIO_HIGH(1, 8)
#define	MREQ_INIT	LPC_IOCON->PIO1_8 = 0x0C0;	\
	 LPC_GPIO1->DIR |= (1<<8);	\
	 MREQ_OFF

#define	IORQ_ON		GPIO_LOW(1, 9)
#define	IORQ_OFF	GPIO_HIGH(1, 9)
#define	IORQ_INIT	LPC_IOCON->PIO1_9 = 0x0C0;	\
	 LPC_GPIO1->DIR |= (1<<9);	\
	 IORQ_OFF

#define	RD_ON		GPIO_LOW(1, 10)
#define	RD_OFF		GPIO_HIGH(1, 10)
#define	RD_INIT		LPC_IOCON->PIO1_10 = 0x0C0;	\
	 LPC_GPIO1->DIR |= (1<<10);	\
	 RD_OFF

#define	WR_ON		GPIO_LOW(1, 11)
#define	WR_OFF		GPIO_HIGH(1, 11)
#define	WR_INIT		LPC_IOCON->PIO1_11 = 0x0C0;	\
	 LPC_GPIO1->DIR |= (1<<11);	\
	 WR_OFF

#define	DATA_DIR_IN		LPC_GPIO2->DIR &= ~0x00FF
#define	DATA_DIR_OUT	LPC_GPIO2->DIR |= 0x00FF
#define	IS_DATA_DIR_OUT	(LPC_GPIO2->DIR & 0x00FF)
#define	DATA_OUT(d)		LPC_GPIO2->MASKED_ACCESS[0x00FF] = d
#define	DATA_IN			((uint8_t)(LPC_GPIO2->MASKED_ACCESS[0x00FF]))
#define	DATA_INIT	\
	 LPC_IOCON->PIO2_0 = 0x0F0;	\
	 LPC_IOCON->PIO2_1 = 0x0F0;	\
	 LPC_IOCON->PIO2_2 = 0x0F0;	\
	 LPC_IOCON->PIO2_3 = 0x0F0;	\
	 LPC_IOCON->PIO2_4 = 0x0F0;	\
	 LPC_IOCON->PIO2_5 = 0x0F0;	\
	 LPC_IOCON->PIO2_6 = 0x0F0;	\
	 LPC_IOCON->PIO2_7 = 0x0F0;	\
	 DATA_DIR_IN

#define	ADRS_DIR_IN	\
	 LPC_GPIO1->DIR &= ~0x003F;	\
	 LPC_GPIO2->DIR &= ~0x0F00;	\
	 LPC_GPIO3->DIR &= ~0x003F
#define	ADRS_DIR_OUT	\
	 LPC_GPIO1->DIR |= 0x003F;	\
	 LPC_GPIO2->DIR |= 0x0F00;	\
	 LPC_GPIO3->DIR |= 0x003F
#define	ADRS_OUT(d)	\
	 LPC_GPIO1->MASKED_ACCESS[0x003F] = d;	\
	 LPC_GPIO2->MASKED_ACCESS[0x0F00] = (d << 2);	\
	 LPC_GPIO3->MASKED_ACCESS[0x003F] = (d >> 10)
#define	ADRS_INIT	\
	 LPC_IOCON->R_PIO1_0 = 0x0C1;	\
	 LPC_IOCON->R_PIO1_1 = 0x0C1;	\
	 LPC_IOCON->R_PIO1_2 = 0x0C1;	\
	 LPC_IOCON->SWDIO_PIO1_3 = 0x0C1;	\
	 LPC_IOCON->PIO1_4 = 0x0C0;	\
	 LPC_IOCON->PIO1_5 = 0x0C0;	\
	 LPC_IOCON->PIO2_8 = 0x0C0;	\
	 LPC_IOCON->PIO2_9 = 0x0C0;	\
	 LPC_IOCON->PIO2_10 = 0x0C0;	\
	 LPC_IOCON->PIO2_11 = 0x0C0;	\
	 LPC_IOCON->PIO3_0 = 0x0C0;	\
	 LPC_IOCON->PIO3_1 = 0x0C0;	\
	 LPC_IOCON->PIO3_2 = 0x0C0;	\
	 LPC_IOCON->PIO3_3 = 0x0C0;	\
	 LPC_IOCON->PIO3_4 = 0x0C0;	\
	 LPC_IOCON->PIO3_5 = 0x0C0;	\
	 ADRS_DIR_OUT
	 


/* Externals */
extern void initialise_monitor_handles();

#ifdef __cplusplus
}
#endif

#endif  /* __HW_CONFIG_H */
