/********************************************************************************/
/*!
	@file			startup_LPC11xx.s
	@author         Nemui Trinomius (http://nemuisan.blog.bai.ne.jp)
    @version        1.00
    @date           2014.01.14
	@brief          LPC1114/LPC1115's Startup ASM Code						@n
					Based On STM32F107VCT6's Startup

    @section HISTORY
		2014.01.14	V1.00	Start Here.

    @section LICENSE
		BSD License. See Copyright.txt
*/
/********************************************************************************/

	.cpu cortex-m0
	.fpu softvfp
	.align	1
	.code	16
	.thumb_func

.global	g_pfnVectors
.global	Default_Handler

/* start address for the initialization values of the .data section. 
defined in linker script */
.word	_sidata
/* start address for the .data section. defined in linker script */  
.word	_sdata
/* end address for the .data section. defined in linker script */
.word	_edata
/* start address for the .bss section. defined in linker script */
.word	_sbss
/* end address for the .bss section. defined in linker script */
.word	_ebss

/* Calculated Vector Checksum   */
/* See Also Linker Script Files */
.word	_vectsum

/* LPC1114 Code Read Protection          */
/* 0x11223344 is NO CRP Protection Level */
.section .crp
crp_word:		.word	0x11223344

/**
 * @brief  This is the code that gets called when the processor first
 *          starts execution following a reset event. Only the absolutely
 *          necessary set is performed, after which the application
 *          supplied main() routine is called. 
 * @param  None
 * @retval : None
*/

    .section	.boot,"ax",%progbits
	.weak	Reset_Handler
	.type	Reset_Handler, %function
Reset_Handler:	

/* Copy Initial *(.data*) to SRAM */
 		ldr		r3, =_sdata
 		ldr		r2, =_etext
 		b		LoopCopyDataInit
CopyDataInit:
 		ldrb	r1, [r2]
 		add		r2, r2, #1
 		strb	r1, [r3]
 		add		r3, r3, #1
LoopCopyDataInit:
 		ldr		r1, =_edata
 		cmp		r3, r1
 		bcc		CopyDataInit

/* Clear *(.bss*) Region */
 		ldr		r3, =_sbss
 		b		LoopFillZerobss
FillZerobss:
 		mov		r2, #0
 		strb	r2, [r3]
 		add		r3, r3, #1
LoopFillZerobss:
 		ldr		r2, =_ebss
 		cmp		r3, r2
 		bcc		FillZerobss

/* Call the clock system intitialization function.*/
		bl  SystemInit  
/* Call the application's entry point.*/
		bl	main
		bx	lr   

.size	Reset_Handler, .-Reset_Handler

/**
 * @brief  This is the code that gets called when the processor receives an 
 *         unexpected interrupt.  This simply enters an infinite loop, preserving
 *         the system state for examination by a debugger.
 * @param  None     
 * @retval None       
*/
    .section	.text.Default_Handler,"ax",%progbits
Default_Handler:
Infinite_Loop:
	b	Infinite_Loop
	.size	Default_Handler, .-Default_Handler
/******************************************************************************
*
* The minimal vector table for a Cortex M0.  Note that the proper constructs
* must be placed on this to ensure that it ends up at physical address
* 0x0000.0000.
*
******************************************************************************/
 	.section .isr_vector,"a",%progbits
	.type	g_pfnVectors, %object
	.size	g_pfnVectors, .-g_pfnVectors

g_pfnVectors:
	.word	_estack						/* 00:Cortex-M0 Initial Stack Pointer 			*/
	.word	Reset_Handler				/* 01:Cortex-M0 Exeption Reset 					*/
	.word	NMI_Handler					/* 02:Cortex-M0 Exeption Non Maskable Interrupt */
	.word	HardFault_Handler			/* 03:Cortex-M0 Exeption HardFault				*/
	.word	0							/* 04:Reserved									*/
	.word	0							/* 05:Reserved									*/
	.word	0							/* 06:Reserved									*/
	.word	_vectsum					/* 07:NxP ARM-uC's Vector Checksum Validation 	*/
	.word	0							/* 08:Reserved									*/
	.word	0							/* 09:Reserved									*/
	.word	0							/* 10:Reserved									*/
	.word	SVC_Handler					/* 11:Cortex-M0 Exeption SVC Call			    */
	.word	0							/* 12:Reserved									*/
	.word	0							/* 13:Reserved									*/
	.word	PendSV_Handler				/* 14:Cortex-M0 Exeption PendSV Interrupt 		*/
	.word	SysTick_Handler				/* 15:Cortex-M0 Exeption SysTick Interrupt 		*/

    /* External Interrupts */
	.word   WAKEUP_IRQHandler           /* 16:PIO0_0  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 17:PIO0_1  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 18:PIO0_2  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 19:PIO0_3  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 20:PIO0_4  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 21:PIO0_5  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 22:PIO0_6  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 23:PIO0_7  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 24:PIO0_8  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 25:PIO0_9  Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 26:PIO0_10 Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 27:PIO0_11 Wakeup Interrupt 					*/
	.word   WAKEUP_IRQHandler           /* 28:PIO1_0  Wakeup Interrupt 					*/
	.word   CAN_IRQHandler              /* 29:C_CAN Interrupt            		 		*/
	.word   SSP1_IRQHandler             /* 30:SPI/SSP1 Interrupt            		 	*/
	.word   I2C_IRQHandler              /* 31:I2C Interrupt	            		 		*/
	.word   TIMER16_0_IRQHandler        /* 32:16-bit Timer0 Interrupt            		*/
	.word   TIMER16_1_IRQHandler        /* 33:16-bit Timer1 Interrupt              		*/
	.word   TIMER32_0_IRQHandler        /* 34:32-bit Timer0 Interrupt              		*/
	.word   TIMER32_1_IRQHandler        /* 35:32-bit Timer1 Interrupt            		*/
	.word   SSP0_IRQHandler             /* 36:SSP0 Interrupt               		 		*/
	.word   UART_IRQHandler             /* 37:UART Interrupt               		 		*/
	.word	0							/* 38:Reserved									*/
	.word	0							/* 39:Reserved									*/
	.word   ADC_IRQHandler              /* 40:A/D Converter Interrupt            	  	*/
	.word   WDT_IRQHandler              /* 41:Watchdog Timer Interrupt              	*/
	.word   BOD_IRQHandler              /* 42:BOD Interrupt              				*/
	.word	0							/* 43:Reserved									*/
	.word   PIOINT3_IRQHandler          /* 44:GPIO interrupt status of port 3           */
	.word   PIOINT2_IRQHandler          /* 45:GPIO interrupt status of port 2           */
	.word   PIOINT1_IRQHandler          /* 46:GPIO interrupt status of port 1           */
	.word   PIOINT0_IRQHandler          /* 47:GPIO interrupt status of port 0           */
   
/* Exception Handlers */

    .weak   NMI_Handler
    .type   NMI_Handler, %function
NMI_Handler:
    B       .
    .size   NMI_Handler, . - NMI_Handler

    .weak   HardFault_Handler
    .type   HardFault_Handler, %function
HardFault_Handler:
    B       .
    .size   HardFault_Handler, . - HardFault_Handler

    .weak   SVC_Handler
    .type   SVC_Handler, %function
SVC_Handler:
    B       .
    .size   SVC_Handler, . - SVC_Handler

    .weak   PendSV_Handler
    .type   PendSV_Handler, %function
PendSV_Handler:
    B       .
    .size   PendSV_Handler, . - PendSV_Handler

    .weak   SysTick_Handler
    .type   SysTick_Handler, %function
SysTick_Handler:
    B       .
    .size   SysTick_Handler, . - SysTick_Handler


/* IRQ Handlers */

    .macro  IRQ handler
    .weak   \handler
    .set    \handler, Default_Handler
    .endm

	IRQ		WAKEUP_IRQHandler
	IRQ		CAN_IRQHandler
	IRQ		SSP1_IRQHandler
	IRQ		I2C_IRQHandler
	IRQ		TIMER16_0_IRQHandler
	IRQ		TIMER16_1_IRQHandler
	IRQ		TIMER32_0_IRQHandler
	IRQ		TIMER32_1_IRQHandler
	IRQ		SSP0_IRQHandler
	IRQ		UART_IRQHandler
	IRQ		ADC_IRQHandler
	IRQ		WDT_IRQHandler
	IRQ		BOD_IRQHandler
	IRQ		PIOINT3_IRQHandler
	IRQ		PIOINT2_IRQHandler
	IRQ		PIOINT1_IRQHandler
	IRQ		PIOINT0_IRQHandler

    .end
