//*******************************************************
//
//	Z80EM1
//
//	シリアルポート制御
//
//	2014.9.15	KAWAKAMI Yukio
//*******************************************************

#include	<stdio.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<stdbool.h>
#include	<ctype.h>

#include	"hw_config.h"
#include	"uart.h"
#include	"com.h"

#if TX_BUFF_SIZE > 0
#define	ENABLE_TRX_INT	LPC_UART->IER |= IER_RBR | IER_THRE
#else
#define	ENABLE_TRX_INT	LPC_UART->IER |= IER_RBR
#endif
#define	DISABLE_TRX_INT	LPC_UART->IER = 0;

COM_WORK	Com0;
COM_WORK	*UART0_Com;

//*******************************************************
//	上位レベル関数
//*******************************************************
void putch(uint8_t data){

	write_com(&Com0, data);
}


uint8_t getch(void){

	if (check_com(&Com0)){
		return read_com(&Com0);
	} else {
		return 0;
	}
}

uint8_t keypressed(void){

	if (check_com(&Com0)){
		return 1;
	} else {
		return 0;
	}
}


void cputs(const char *s){
	char	c;

	while((c = *s++)){
		if (c == '\n'){
			putch('\r');
		}
		putch(c);
	}
}


void cgets(char *s, int bufsize){
	char *p;
	int c;

	memset(s, 0, bufsize);

	p = s;

	for (p = s; p < s + bufsize-1;){
		/* 20090521Nemui */
		do{		
			c = getch();
		} while(c == false);
		/* 20090521Nemui */
		switch (c)
			{
			  case '\r' :
			  case '\n' :
				putch('\r');
				putch('\n');
				*p = '\n';
				return;

			  case '\b' :
				if (p > s)
					{
						*p-- = 0;
						putch('\b');
						putch(' ');
						putch('\b');
					}
				break;

			  default :
				putch(c);
				*p++ = c;
				break;
			}
	}

	return;
}


//*******************************************************
//	シリアルポート初期化
//*******************************************************
void init_com(COM_WORK *com, uint32_t baud){
	uint32_t Fdiv;
	uint32_t regVal;

	NVIC_DisableIRQ(UART_IRQn);
	memset((void *)com, 0, sizeof(COM_WORK));
	UART0_Com = com;

	LPC_IOCON->PIO1_6 = 0x0F1;     // RXD pull up
	LPC_IOCON->PIO1_7 = 0x0C1;     // TXD
	/* Enable UART clock */
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12);
	LPC_SYSCON->UARTCLKDIV = 0x1;     /* divided by 1 */

	LPC_UART->LCR = 0x83;             /* 8 bits, no Parity, 1 Stop bit */
	regVal = LPC_SYSCON->UARTCLKDIV;

	if ((baud == 115200)&&(SystemCoreClock == 12000000)){
		// 115200bps @12MHz
		LPC_UART->DLM = 0;
		LPC_UART->DLL = 4;
		LPC_UART->FDR = (8 << 4)|(5 << 0);	// MULVAL=8, DIVADDVAL=5
	} else {
		Fdiv = (((SystemCoreClock * LPC_SYSCON->SYSAHBCLKDIV) / regVal) / 16);
		Fdiv += baud / 2;
		Fdiv /= baud;
		LPC_UART->DLM = Fdiv / 256;							
		LPC_UART->DLL = Fdiv % 256;
		LPC_UART->FDR = (1 << 4);		// MULVAL=1
	}

	LPC_UART->LCR = 0x03;		/* DLAB = 0 */
	LPC_UART->FCR = 0x07;		/* Enable and reset TX and RX FIFO. */

	/* Read to clear the line status. */
	regVal = LPC_UART->LSR;

	/* Ensure a clean start, no data in either TX or RX FIFO. */
	// CodeRed - added parentheses around comparison in operand of &
	while (( LPC_UART->LSR & (LSR_THRE|LSR_TEMT)) != (LSR_THRE|LSR_TEMT) );
	while ( LPC_UART->LSR & LSR_RDR ){
		regVal = LPC_UART->RBR;	/* Dump data from RX FIFO */
	}
 
	/* Enable the UART Interrupt */
	NVIC_EnableIRQ(UART_IRQn);
	LPC_UART->IER = IER_RBR;	/* Enable UART interrupt */
}


//*******************************************************
//	シリアルデータ送信完了まで待つ
//*******************************************************
void wait_send_com(COM_WORK *com){

	// 送信バッファが空になるまで待つ
	while(com->obuff_remain)	SLEEP;
	// 更に送信完了まで待つ
	while(!(LPC_UART->LSR & LSR_TEMT));
}


//*******************************************************
//	シリアルポート受信バッファチェック
//*******************************************************
int check_com(COM_WORK *com){

	return com->ibuff_remain;
}


//*******************************************************
//	シリアルポートデータ受信
//*******************************************************
uint8_t read_com(COM_WORK *com){
	uint8_t	data;

	// 受信データが無ければ来るまで待つ
	while(!com->ibuff_remain){
		WDR;
		SLEEP;
	}
	
	// 受信バッファからデータ読み出し
	data = com->ibuff[com->ibuff_out++];
	if (com->ibuff_out >= sizeof(com->ibuff)){
		com->ibuff_out = 0;
	}
	DISABLE_TRX_INT;	// 送受信割り込み禁止
	--com->ibuff_remain;
	ENABLE_TRX_INT;		// 送受信割り込み許可
	
	return data;
}

//*******************************************************
//	シリアルポートデータ送信
//*******************************************************
void write_com(COM_WORK *com, uint8_t data){

#if TX_BUFF_SIZE > 0
	// 送信バッファがいっぱいなら待つ
	while(com->obuff_remain >= TX_BUFF_SIZE){
		WDR;
		SLEEP;
	}

	DISABLE_TRX_INT;	// 送受信割り込み禁止
	if ((com->obuff_remain == 0)&&(LPC_UART->LSR & LSR_THRE)){
		LPC_UART->THR = data;
	} else {
		// データを送信バッファに積む
		com->obuff[com->obuff_in++] = data;
		if (com->obuff_in >= sizeof(com->obuff)){
			com->obuff_in = 0;
		}
		com->obuff_remain++;
	}
	ENABLE_TRX_INT;		// 送受信割り込み許可
#else
	while(!(LPC_UART->LSR & LSR_THRE)){
		WDR;
		SLEEP;
	}
	LPC_UART->THR = data;
#endif
}


//*******************************************************
//	文字列送信
//*******************************************************
void write_com_str(COM_WORK *com, char *str){
	char	c;

	while((c = *str++)){
		write_com(com, (uint8_t)c);
	}
}


//*******************************************************
//	UART 割り込みハンドラ
//*******************************************************
void UART_IRQHandler(void){

#if TX_BUFF_SIZE > 0
	if (LPC_UART->IER & IER_THRE){
		while((LPC_UART->LSR & LSR_THRE)&&(UART0_Com->obuff_remain)){
			LPC_UART->THR = UART0_Com->obuff[UART0_Com->obuff_out++];
			if (UART0_Com->obuff_out >= TX_BUFF_SIZE){
				UART0_Com->obuff_out = 0;
			}
			--(UART0_Com->obuff_remain);
		}
		if (UART0_Com->obuff_remain == 0){
			LPC_UART->IER &= ~IER_THRE;	// 送信割り込み禁止
		}
	}
#endif

	if (LPC_UART->IER & IER_RBR){
		while(LPC_UART->LSR & LSR_RDR){
			UART0_Com->ibuff[UART0_Com->ibuff_in++] = LPC_UART->RBR;
			if (UART0_Com->ibuff_in >= RX_BUFF_SIZE){
				UART0_Com->ibuff_in = 0;
			}
			(UART0_Com->ibuff_remain)++;
		}
	}
}


